package com.example.gestionstagesbackend.controllers;

import com.example.gestionstagesbackend.model.User;
import com.example.gestionstagesbackend.services.UserService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.HttpSession;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping("/api/auth")
@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
public class AuthController {

    private final UserService userService;
    private final AuthenticationManager authenticationManager;

    public AuthController(UserService userService, AuthenticationManager authenticationManager) {
        this.userService = userService;
        this.authenticationManager = authenticationManager;
    }

    /** REGISTER NEW USER **/
    @PostMapping("/register")
    public ResponseEntity<?> registerUser(@RequestBody User user) {
        if (userService.findByUsername(user.getUsername()).isPresent()) {
            System.out.println("❌ Registration failed: Username " + user.getUsername() + " is already taken.");
            return ResponseEntity.badRequest().body(Map.of("message", "Username is already taken"));
        }

        // Assign default role if none is provided
        if (user.getRoles() == null || user.getRoles().isEmpty()) {
            user.setRoles(Collections.singleton("ROLE_ETUDIANT")); // Default role
        }

        // Save user with hashed password
        User newUser = userService.registerUser(user);
        System.out.println("✅ User registered successfully: " + newUser.getUsername());
        return ResponseEntity.ok("User registered successfully");
    }

    /** LOGIN USER (No JWT, Just Plain Text Response) **/
    @PostMapping("/login")
    public ResponseEntity<?> loginUser(@RequestBody Map<String, String> loginRequest) {
        String username = loginRequest.get("username");
        String password = loginRequest.get("password");

        Optional<User> user = userService.findByUsername(username);
        if (user.isEmpty()) {
            return ResponseEntity.status(401).body("Invalid credentials");
        }

        try {
            Authentication authentication = authenticationManager.authenticate(
                    new UsernamePasswordAuthenticationToken(username, password)
            );

            SecurityContextHolder.getContext().setAuthentication(authentication);

            // ✅ Send username & familyname in the response
            Map<String, Object> responseBody = new HashMap<>();
            responseBody.put("id", user.get().getId()); // 🔥 Ajoute l'ID ici
            responseBody.put("username", user.get().getUsername());
            responseBody.put("familyname", user.get().getFamilyname()); // Assure-toi que ce champ existe bien
            responseBody.put("roles", user.get().getRoles());
            responseBody.put("message", "Login successful");

            return ResponseEntity.ok(responseBody);
        } catch (Exception e) {
            return ResponseEntity.status(401).body("Invalid username or password");
        }
    }

    /** LOGOUT USER **/
    @PostMapping("/logout")
    public ResponseEntity<?> logoutUser(HttpServletRequest request, HttpServletResponse response) {
        System.out.println("🔓 Logging out user: " + SecurityContextHolder.getContext().getAuthentication());

        // Invalidate session
        HttpSession session = request.getSession(false);
        if (session != null) {
            session.invalidate();
        }

        // Clear authentication
        SecurityContextHolder.clearContext();

        System.out.println("✅ Logout successful.");
        return ResponseEntity.ok("Logout successful");
    }

}
