package com.example.gestionstagesbackend.controllers;

import com.example.gestionstagesbackend.model.Candidacy;
import com.example.gestionstagesbackend.model.Student;
import com.example.gestionstagesbackend.model.Stage;
import com.example.gestionstagesbackend.model.UnivSupervisor;
import com.example.gestionstagesbackend.services.CandidacyService;
import com.example.gestionstagesbackend.services.StudentService;
import com.example.gestionstagesbackend.services.StageService;
import com.example.gestionstagesbackend.services.UnivSupervisorService;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/candidacies")
@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
public class CandidacyController {
    private final CandidacyService candidacyService;
    private final StudentService studentService;
    private final StageService stageService;
    private final UnivSupervisorService univSupervisorService;

    public CandidacyController(CandidacyService candidacyService, StudentService studentService,
                               StageService stageService, UnivSupervisorService univSupervisorService) {
        this.candidacyService = candidacyService;
        this.studentService = studentService;
        this.stageService = stageService;
        this.univSupervisorService = univSupervisorService;
    }

    //@PreAuthorize("hasRole('ETUDIANT') or hasRole('ADMIN')")
    @PostMapping("/add")
    public ResponseEntity<?> createCandidacy(@RequestBody Candidacy candidacy) {
        if (candidacy.getStudent() == null || candidacy.getStudent().getId() == null) {
            return ResponseEntity.badRequest().body("Student ID is required");
        }
        if (candidacy.getStage() == null || candidacy.getStage().getId() == null) {
            return ResponseEntity.badRequest().body("Stage ID is required");
        }
        if (candidacy.getUnivSupervisor() != null && candidacy.getUnivSupervisor().getId() == null) {
            return ResponseEntity.badRequest().body("UnivSupervisor ID is invalid");
        }

        Optional<Student> student = studentService.getStudentById(candidacy.getStudent().getId());
        Optional<Stage> stage = stageService.getStageById(candidacy.getStage().getId());
        Optional<UnivSupervisor> univSupervisor = Optional.empty();

        if (candidacy.getUnivSupervisor() != null) {
            univSupervisor = univSupervisorService.getUnivSupervisorById(candidacy.getUnivSupervisor().getId());
            if (univSupervisor.isEmpty()) {
                return ResponseEntity.badRequest().body("UnivSupervisor not found");
            }
        }

        if (student.isEmpty()) {
            return ResponseEntity.badRequest().body("Student not found");
        }
        if (stage.isEmpty()) {
            return ResponseEntity.badRequest().body("Stage not found");
        }

        candidacy.setStudent(student.get());
        candidacy.setStage(stage.get());
        univSupervisor.ifPresent(candidacy::setUnivSupervisor);
        if (candidacy.getComment() == null) {
            candidacy.setComment(""); // Valeur par défaut si vide
        }

        Candidacy savedCandidacy = candidacyService.saveCandidacy(candidacy);
        return ResponseEntity.ok(savedCandidacy);
    }

    //@PreAuthorize("hasRole('ETUDIANT') or hasRole('ADMIN')")
    @GetMapping
    public ResponseEntity<List<Candidacy>> getAllCandidacies() {
        return ResponseEntity.ok(candidacyService.getAllCandidacies());
    }

    //@PreAuthorize("hasRole('ETUDIANT') or hasRole('ADMIN')")
    @GetMapping("/{id}")
    public ResponseEntity<Candidacy> getCandidacyById(@PathVariable Long id) {
        Optional<Candidacy> optionalCandidacy = candidacyService.getCandidacyById(id);

        if (optionalCandidacy.isPresent()) {
            Candidacy candidacy = optionalCandidacy.get();
            candidacy.getStage().getId(); // Ensure Stage ID is included
            return ResponseEntity.ok(candidacy);
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    //@PreAuthorize("hasRole('ETUDIANT') or hasRole('ADMIN')")
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteCandidacy(@PathVariable Long id) {
        if (!candidacyService.existsById(id)) {
            return ResponseEntity.notFound().build();
        }
        candidacyService.deleteCandidacy(id);
        return ResponseEntity.noContent().build();
    }

    //@PreAuthorize("hasRole('ETUDIANT') or hasRole('ADMIN')")
    @PutMapping("/{id}")
    public ResponseEntity<?> updateCandidacy(@PathVariable Long id, @RequestBody Candidacy updatedCandidacy) {
        Optional<Candidacy> existingCandidacy = candidacyService.getCandidacyById(id);

        if (existingCandidacy.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        Candidacy candidacy = existingCandidacy.get();

        // Update Student
        if (updatedCandidacy.getStudent() != null && updatedCandidacy.getStudent().getId() != null) {
            candidacy.setStudent(updatedCandidacy.getStudent());
        }

        // Update Stage
        if (updatedCandidacy.getStage() != null && updatedCandidacy.getStage().getId() != null) {
            candidacy.setStage(updatedCandidacy.getStage());
        }

        // Update UnivSupervisor
        if (updatedCandidacy.getUnivSupervisor() != null) {
            candidacy.setUnivSupervisor(updatedCandidacy.getUnivSupervisor());
        }

        // Update Request State (FirstRequest, Accepted, Declined)
        if (updatedCandidacy.getRequestState() != null) {
            candidacy.setRequestState(updatedCandidacy.getRequestState());
        }

        // Update Accepted State (wait, inProgress, terminated)
        if (updatedCandidacy.getAcceptedState() != null) {
            candidacy.setAcceptedState(updatedCandidacy.getAcceptedState());
        }
        if (updatedCandidacy.getComment() != null) {
            candidacy.setComment(updatedCandidacy.getComment());
        }

        // Save the updated candidacy
        Candidacy savedCandidacy = candidacyService.saveCandidacy(candidacy);
        return ResponseEntity.ok(savedCandidacy);
    }




}
