package com.example.gestionstagesbackend.services;

import com.example.gestionstagesbackend.model.User;
import com.example.gestionstagesbackend.repositories.UserRepository;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Optional;
import java.util.Set;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;

    public UserService(UserRepository userRepository, PasswordEncoder passwordEncoder) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
    }

    public boolean checkPassword(String rawPassword, String encodedPassword) {
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }

    public User registerUser(User user) {
        if (user.getPassword() == null || user.getPassword().trim().isEmpty()) {
            throw new IllegalArgumentException("Password cannot be null or empty");
        }
        user.setPassword(passwordEncoder.encode(user.getPassword())); // Hash password before saving
        return userRepository.save(user);
    }


    public Optional<User> findByUsername(String username) {
        return userRepository.findByUsername(username);
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        User user = userRepository.findByUsername(username)
                .orElseThrow(() -> new UsernameNotFoundException("User not found: " + username));

        // Convert to UserDetails for Spring Security
        return org.springframework.security.core.userdetails.User
                .withUsername(user.getUsername())
                .password(user.getPassword())
                .authorities(user.getRoles().toArray(new String[0])) // Convert roles to authorities
                .build();
    }

    @PostConstruct
    public void createDefaultUsers() {
        createUserIfNotExists("admin", "admin123", "Admin", "ROLE_ADMIN");
        createUserIfNotExists("etudiant1", "etudiant123", "Etudiant1", "ROLE_ETUDIANT");
        createUserIfNotExists("etudiant2", "etudiant123", "Etudiant2", "ROLE_ETUDIANT");
        createUserIfNotExists("entreprise", "entreprise123", "Entreprise", "ROLE_ENTREPRISE");
        createUserIfNotExists("superviseur", "superviseur123", "Superviseur", "ROLE_SUPERVISEUR");
    }

    private void createUserIfNotExists(String username, String password, String familyname, String role) {
        if (userRepository.findByUsername(username).isEmpty()) {
            User user = new User();
            user.setUsername(username);
            user.setPassword(passwordEncoder.encode(password));
            user.setFamilyname(familyname);
            user.setRoles(Set.of(role));
            userRepository.save(user);

            // Si l'utilisateur est un étudiant, on crée un étudiant associé
            if (role.equals("ROLE_ETUDIANT")) {
                studentService.createStudentIfNotExists(username, familyname);
            }

            System.out.println("✅ Created user: " + username + " with role " + role);
        }
    }

    @Autowired
    private StudentService studentService; // Injection du service pour créer un étudiant
}
